using HiTeachCE.Models;
using HiTeachCE.Services;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Extension.DataResult.JsonRpcRequest;
using TEAMModelOS.SDK.Extension.DataResult.JsonRpcResponse;
using TEAMModelOS.SDK.Extension.JwtAuth.Models;

namespace HiTeachCE.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class WeChatController :BaseController
    {
        public WeChatService weChatService;

        public WeChatController(WeChatService weChat) {
            weChatService = weChat;
        }

        /// <summary>
        /// 验证微信登录
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("VerifyWeChat")]
        [AllowAnonymous]
        public async Task<BaseJosnRPCResponse> VerifyWeChat(JosnRPCRequest<Dictionary<string, object>> request)
        {
            JsonRPCResponseBuilder responseBuilder = JsonRPCResponseBuilder.custom();
            object obj = await weChatService.VerifyWeChat(request.@params);
            return responseBuilder.Data(obj).build();
        }
        /// <summary>
        /// 获取微信登录配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("GetWechatConfig")]
        [AllowAnonymous]
        public BaseJosnRPCResponse GetWechatConfig(JosnRPCRequest<string> request)
        {
            JsonRPCResponseBuilder responseBuilder = JsonRPCResponseBuilder.custom();
            Dictionary<string, object> obj = weChatService.GetWechatConfig(request.@params);
            return responseBuilder.Data(obj).build();
        }

        /// <summary>
        /// 验证小程序登录，获取信息
        /// js_code :"  "
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("VerifyMiniAPP")]
        [AllowAnonymous]
        public async Task<BaseJosnRPCResponse> VerifyMiniAPP(JosnRPCRequest<Dictionary<string, object>> request)
        {
            JsonRPCResponseBuilder responseBuilder = JsonRPCResponseBuilder.custom();
            MiniAPPLoginInfo obj = await weChatService.VerifyMiniAPP(request.@params);
            return responseBuilder.Data(obj).build();
        }
        /// <summary>
        /// 获取小程序AccessToken
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("GetMiniAPPAccessToken")]
        [AllowAnonymous]
        public async Task<BaseJosnRPCResponse> GetMiniAPPAccessToken()
        {
            JsonRPCResponseBuilder responseBuilder = JsonRPCResponseBuilder.custom();
            MiniAPPToken obj = await weChatService.GetMiniAPPAccessToken();
            return responseBuilder.Data(obj).build();
        }

        /// <summary>
        /// 小程序后端登录
        /// code 
        /// encryptedData
        /// iv
        /// signature
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("MiniAPPLogin")]
        [AllowAnonymous]
        public async Task<BaseJosnRPCResponse> MiniAPPLogin(JosnRPCRequest<Dictionary<string, object>> request)
        {
            JsonRPCResponseBuilder responseBuilder = JsonRPCResponseBuilder.custom();
            JwtResponse obj = await weChatService.MiniAPPLogin(request.@params);
            return responseBuilder.Data(obj).build();
        }
    }
}
