using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Authentication;
using System.Text;
using System.Threading.Tasks;
using Hei.Captcha;
using HiTeachCE.Extension;
using HiTeachCE.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.WebSockets;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using MQTTnet;
using MQTTnet.AspNetCore;
using MQTTnet.Client.Receiving;
using MQTTnet.Protocol;
using MQTTnet.Server;
using SqlSugar;
using TEAMModelOS.SDK.Context.Configuration;
using TEAMModelOS.SDK.Context.Filter;
using TEAMModelOS.SDK.Extension.JwtAuth;
using static HiTeachCE.Controllers.LoginController;
using VueCliMiddleware;
using Microsoft.AspNetCore.SpaServices;
using HiTeachCE.Extension.Mqtt;
using HiTeachCE.Context;

namespace HiTeachCE
{
    public class Startup
    {
        public Startup(IConfiguration configuration, IWebHostEnvironment env)
        {
            Configuration = configuration;
            BaseConfigModel.SetBaseConfig(Configuration, env.ContentRootPath, env.WebRootPath);
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.Configure<FormOptions>(x =>
            {
                x.ValueLengthLimit = int.MaxValue;
                x.MultipartBodyLengthLimit = long.MaxValue; // In case of multipart
                x.MultipartHeadersLengthLimit = int.MaxValue;
            });
            
            services.AddControllersWithViews();
            //跨域设置
            services.AddCors();
            //設定跨域請求
            //引入Jwt配置
            services.Auth(Configuration.GetSection("JwtSetting"));
            //JWT 黑名单///https://blog.csdn.net/sinat_14899485/article/details/88591848 jwt 黑名单
            //注入CSRedis
            var csredis = new CSRedis.CSRedisClient(Configuration.GetSection("Redis:ConnectionString").Get<string>());
            RedisHelper.Initialization(csredis);
            services.AddHeiCaptcha();
            //https://www.cnblogs.com/JacZhu/p/6837676.html
            // 注册验证要求的处理器，可通过这种方式对同一种要求添加多种验证
            //全局扫描基于IBusinessService接口的实现类
            services.Scan(scan => scan.FromApplicationDependencies()
               .AddClasses(classes => classes.AssignableTo<IBusinessService>())
                   .AsSelfWithInterfaces()
                   .WithScopedLifetime());
            services.AddCors(options =>
            {
                // CorsPolicy 是自訂的 Policy 名稱
                options.AddPolicy("CorsPolicy", policy =>
                {
                    policy.AllowAnyOrigin()
                    //.WithOrigins("http://localhost:5001")
                    .AllowAnyHeader()
                    .AllowAnyMethod();
                  //  .AllowCredentials();
                });
            });
        
            services.MQTTConnection();
            // Add framework services.
            services.AddMvc(options =>
            {
                options.Filters.Add(new ApiExceptionFilterAttribute());
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
               // app.UseExceptionHandler("/Home/Error");
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }

            
          //  app.UseMiddleware<HiTeachCE.Context.HttpGlobalExceptionInvoke>();
            app.UseHttpsRedirection();
            app.UseStaticFiles();

            //SqlSugarClient db = new SqlSugarClient(new ConnectionConfig()
            //{
            //    ConnectionString = BaseConfigModel.Configuration["DbConnection:MySqlConnectionString"],
            //    DbType = DbType.MySql,
            //    IsAutoCloseConnection = true,
            //    InitKeyType = InitKeyType.Attribute
            //});
          //  db.CodeFirst.InitTables(
            //    typeof(ActivationCode),
            //    typeof(Classroom),
            //    typeof(Learner),
              //  typeof(Lecturer)
            //    typeof(Member),
            //    typeof(Organization),
            //    typeof(Subscriber),
            //    typeof(WeChatUser)
              //  );
            app.UseRouting();
            app.UseAuthentication();
            app.UseAuthorization();
            app.UseCors("CorsPolicy"); //使用跨域設定
            
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
                endpoints.MapMqtt("/mqtt");
                //endpoints.MapControllerRoute(
                //    name: "default",
                //    pattern: "{controller=Home}/{action=Index}/{id?}");
#if DEBUG
                endpoints.MapToVueCliProxy(
                    "{*path}",
                    new SpaOptions { SourcePath = "ClientApp", },
                    npmScript: (System.Diagnostics.Debugger.IsAttached) ? "serve" : null
                    );
#else
                endpoints.MapFallbackToFile("index.html");
#endif
            });
            app.UseMqtt();
        }
    }
}
